@@ -5,12 +5,13 @@
 #' @param D Treatment variable
 #' @param level level
 #' @param kernel kernel
+#' @param bw_method bw_method
 #' @param yatchew yatchew
 #' @param placebo placebo
 #' @param dynamic dynamic
 #' @importFrom stats lm qnorm as.formula
 #' @importFrom nprobust lprobust
-#' @import dplyr
+#' @importFrom dplyr %>% group_by mutate cur_group_id ungroup lag lead
 #' @import YatchewTest
 #' @returns Dynamic estimation results (with two periods).
 #' @noRd
@@ -21,6 +22,7 @@ did_had_est <- function(
     D_XX,
     level,
     kernel,
+    bw_method,
     yatchew,
     placebo = FALSE,
     dynamic
@@ -29,12 +31,21 @@ did_had_est <- function(
     df$Y_diff_XX <- df[[Y_XX]]
     df <- subset(df, !is.na(df$Y_diff_XX) & !is.na(df$D_XX))
 
+    # QUG Test
+    D_2_vec <- sort(subset(df, df$D_XX > 0)$D_XX)
+    t_np <- D_2_vec[1]/(D_2_vec[2] - D_2_vec[1])
+    # Test statistic converges in distribution to (E_1/E_2) where E_1 and E_2 are iid random variables from an Exponential(1) distribution
+    # This yields the CDF P(T<t) = (\alpha)/(\alpha + (\beta/t)) where \alpha and \beta are the parameters of the two exponential distributions, so in this case both are 1
+    np_qug_test <- c(t_np, 1-(1/(1+(1/t_np))))
+    names(np_qug_test) <- c("T", "p-value")
+    D_2_vec <- t_np <- NULL
+
     for (j in c(2,3,4)) {
         df[[paste0("D_",j,"_XX")]] <- df$D_XX^j
     }
     df$Y_diff_2_XX <- df$Y_diff_XX^2
 
-    lpres <- lprobust(y = as.vector(df$Y_diff_XX), x = as.vector(df$D_XX), eval = c(0), kernel = kernel)
+    lpres <- lprobust(y = as.vector(df$Y_diff_XX), x = as.vector(df$D_XX), eval = c(0), kernel = kernel, bwselect = bw_method)
 
     ret <- list()
     ret$h_star <- lpres$Estimate[1,2]
@@ -47,7 +58,7 @@ did_had_est <- function(
     ret$G_XX <- max(df$group_XX, na.rm = TRUE)
     mean_Y_diff_XX <- mean(df$Y_diff_XX, na.rm = TRUE)
     if (isTRUE(dynamic)) {
-        mean_D_XX <- mean(df$cumulative_est_XX, na.rm = TRUE)
+        mean_D_XX <- mean(df$cumulative_XX, na.rm = TRUE)
     } else {
         mean_D_XX <- mean(df$D_XX, na.rm = TRUE)
     }
@@ -60,6 +71,7 @@ did_had_est <- function(
     df$count <- as.numeric(df$D_XX <= ret$h_star)
     ret$within_bw_XX <- sum(df$count, na.rm = TRUE)
     df$count <- NULL
+    ret$np_qug_test <- np_qug_test
 
     if (isTRUE(yatchew)) {
         ret$yt_res <- yatchew_test(data = df, Y = "Y_diff_XX", D = "D_XX", het_robust = TRUE, order = 1-as.numeric(placebo))$results
